@@ -56,6 +56,7 @@
 static void     terminal_app_finalize                 (GObject            *object);
 static void     terminal_app_update_accels            (TerminalApp        *app);
 static void     terminal_app_update_mnemonics         (TerminalApp        *app);
+static void     terminal_app_update_windows_accels    (gpointer            user_data);
 static gboolean terminal_app_accel_map_load           (gpointer            user_data);
 static gboolean terminal_app_accel_map_save           (gpointer            user_data);
 static gboolean terminal_app_unset_urgent_bell        (TerminalWindow     *window,
@@ -143,8 +144,9 @@ terminal_app_init (TerminalApp *app)
   terminal_app_update_accels (app);
   terminal_app_update_mnemonics (app);
 
-  /* schedule accel map load */
-  app->accel_map_load_id = g_idle_add_full (G_PRIORITY_LOW, terminal_app_accel_map_load, app, NULL);
+  /* schedule accel map load and update windows when finished */
+  app->accel_map_load_id = g_idle_add_full (G_PRIORITY_LOW, terminal_app_accel_map_load, app,
+                                            terminal_app_update_windows_accels);
 }
 
 
@@ -194,7 +196,9 @@ terminal_app_finalize (GObject *object)
 static void
 terminal_app_update_accels (TerminalApp *app)
 {
-  gboolean no_key;
+  gboolean        no_key;
+  GdkModifierType mod;
+  guint           key;
 
   g_object_get (G_OBJECT (app->preferences),
                 "shortcuts-no-menukey", &no_key,
@@ -203,6 +207,9 @@ terminal_app_update_accels (TerminalApp *app)
                 "gtk-menu-bar-accel",
                 no_key ? NULL : app->initial_menu_bar_accel,
                 NULL);
+  gtk_accelerator_parse (app->initial_menu_bar_accel, &key, &mod);
+  gtk_accel_map_change_entry ("<Actions>/terminal-window/toggle-menubar",
+                              no_key ? 0 : key, no_key ? 0 : mod, TRUE);
 
   g_object_get (G_OBJECT (app->preferences),
                 "shortcuts-no-helpkey", &no_key,
@@ -228,6 +235,20 @@ terminal_app_update_mnemonics (TerminalApp *app)
 
 
 
+static void
+terminal_app_update_windows_accels (gpointer user_data)
+{
+  TerminalApp *app = TERMINAL_APP (user_data);
+  GSList      *lp;
+
+  for (lp = app->windows; lp != NULL; lp = lp->next)
+    terminal_window_rebuild_tabs_menu (TERMINAL_WINDOW (lp->data));
+
+  app->accel_map_load_id = 0;
+}
+
+
+
 static gboolean
 terminal_app_accel_map_save (gpointer user_data)
 {
@@ -274,8 +295,6 @@ terminal_app_accel_map_load (gpointer user_data)
   gchar        name[50];
   guint        i;
 
-  app->accel_map_load_id = 0;
-
   path = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, ACCEL_MAP_PATH);
   if (G_LIKELY (path != NULL))
     {
@@ -383,13 +402,14 @@ terminal_app_create_window (TerminalApp       *app,
 static GtkWidget*
 terminal_app_create_drop_down (TerminalApp        *app,
                                const gchar        *role,
+                               const gchar        *icon,
                                gboolean            fullscreen,
                                TerminalVisibility  menubar,
                                TerminalVisibility  toolbar)
 {
   GtkWidget *window;
 
-  window = terminal_window_dropdown_new (role, fullscreen, menubar, toolbar);
+  window = terminal_window_dropdown_new (role, icon, fullscreen, menubar, toolbar);
 
   terminal_app_take_window (app, GTK_WINDOW (window));
 
@@ -712,6 +732,7 @@ terminal_app_open_window (TerminalApp        *app,
           /* create new drop-down window */
           window = terminal_app_create_drop_down (app,
                                                   attr->role,
+                                                  attr->icon,
                                                   attr->fullscreen,
                                                   attr->menubar,
                                                   attr->toolbar);
@@ -761,7 +782,7 @@ terminal_app_open_window (TerminalApp        *app,
         }
     }
 
-  TERMINAL_WINDOW (window)->scrollbar_visibility = attr->scrollbar;
+  terminal_window_set_scrollbar_visibility (TERMINAL_WINDOW (window), attr->scrollbar);
 
   /* font and zoom for new window */
   if (!reuse_window)
@@ -821,8 +842,7 @@ terminal_app_open_window (TerminalApp        *app,
               if (G_LIKELY (active_terminal != NULL))
                 {
                   /* save window geometry to prevent overriding */
-                  TERMINAL_WINDOW (window)->grid_height = height;
-                  TERMINAL_WINDOW (window)->grid_width = width;
+                  terminal_window_set_grid_size (TERMINAL_WINDOW (window), width, height);
 
                   terminal_screen_force_resize_window (active_terminal, GTK_WINDOW (window),
                                                        width, height);
