@@ -50,16 +50,20 @@ enum
   PROP_BACKGROUND_IMAGE_SHADING,
   PROP_BINDING_BACKSPACE,
   PROP_BINDING_DELETE,
+  PROP_BINDING_AMBIGUOUS_WIDTH,
   PROP_COLOR_FOREGROUND,
   PROP_COLOR_BACKGROUND,
   PROP_COLOR_BACKGROUND_VARY,
   PROP_COLOR_CURSOR,
   PROP_COLOR_SELECTION,
+  PROP_COLOR_SELECTION_BG,
   PROP_COLOR_SELECTION_USE_DEFAULT,
   PROP_COLOR_BOLD,
   PROP_COLOR_BOLD_USE_DEFAULT,
   PROP_COLOR_PALETTE,
   PROP_COMMAND_LOGIN_SHELL,
+  PROP_RUN_CUSTOM_COMMAND,
+  PROP_CUSTOM_COMMAND,
   PROP_DROPDOWN_ANIMATION_TIME,
   PROP_DROPDOWN_KEEP_OPEN_DEFAULT,
   PROP_DROPDOWN_KEEP_ABOVE,
@@ -74,6 +78,7 @@ enum
   PROP_ENCODING,
   PROP_FONT_ALLOW_BOLD,
   PROP_FONT_NAME,
+  PROP_FONT_USE_SYSTEM,
   PROP_MISC_ALWAYS_SHOW_TABS,
   PROP_MISC_BELL,
   PROP_MISC_BELL_URGENT,
@@ -93,9 +98,11 @@ enum
   PROP_MISC_TAB_POSITION,
   PROP_MISC_HIGHLIGHT_URLS,
   PROP_MISC_MIDDLE_CLICK_OPENS_URI,
+  PROP_MISC_COPY_ON_SELECT,
   PROP_MISC_DEFAULT_WORKING_DIR,
   PROP_MISC_REWRAP_ON_RESIZE,
   PROP_MISC_SHIFT_ARROWS_SCROLL,
+  PROP_MISC_SLIM_TABS,
   PROP_SCROLLING_BAR,
   PROP_SCROLLING_LINES,
   PROP_SCROLLING_ON_OUTPUT,
@@ -283,6 +290,8 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
     g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_TITLE, transform_string_to_enum);
   if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_ERASE_BINDING))
     g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_ERASE_BINDING, transform_string_to_enum);
+  if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_AMBIGUOUS_WIDTH_BINDING))
+    g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_AMBIGUOUS_WIDTH_BINDING, transform_string_to_enum);
   if (!g_value_type_transformable (G_TYPE_STRING, TERMINAL_TYPE_CURSOR_SHAPE))
     g_value_register_transform_func (G_TYPE_STRING, TERMINAL_TYPE_CURSOR_SHAPE, transform_string_to_enum);
 
@@ -365,6 +374,17 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                          TERMINAL_ERASE_BINDING_AUTO,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:binding-ambiguous-width:
+   **/
+  preferences_props[PROP_BINDING_AMBIGUOUS_WIDTH] =
+      g_param_spec_enum ("binding-ambiguous-width",
+                         NULL,
+                         "BindingAmbiguousWidth",
+                         TERMINAL_TYPE_AMBIGUOUS_WIDTH_BINDING,
+                         TERMINAL_AMBIGUOUS_WIDTH_BINDING_NARROW,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:color-foreground:
    **/
@@ -415,6 +435,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                            "#ffffff",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:color-selection-bg:
+   **/
+  preferences_props[PROP_COLOR_SELECTION_BG] =
+      g_param_spec_string ("color-selection-bg",
+                           NULL,
+                           "ColorSelectionBackground",
+                           "#ffffff",
+                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:color-selection-use-default:
    **/
@@ -500,6 +530,26 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:run-custom-command:
+   **/
+  preferences_props[PROP_RUN_CUSTOM_COMMAND] =
+      g_param_spec_boolean ("run-custom-command",
+                            NULL,
+                            "RunCustomCommand",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * TerminalPreferences:custom-command:
+   **/
+  preferences_props[PROP_CUSTOM_COMMAND] =
+      g_param_spec_string ("custom-command",
+                           NULL,
+                           "CustomCommand",
+                           "",
+                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:dropdown-animation-time:
    **/
@@ -643,6 +693,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                            "Monospace 12",
                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:font-allow-bold:
+   **/
+  preferences_props[PROP_FONT_USE_SYSTEM] =
+      g_param_spec_boolean ("font-use-system",
+                            NULL,
+                            "FontUseSystem",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:misc-always-show-tabs:
    **/
@@ -835,6 +895,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:misc-copy-on-select:
+   **/
+  preferences_props[PROP_MISC_COPY_ON_SELECT] =
+      g_param_spec_boolean ("misc-copy-on-select",
+                            NULL,
+                            "MiscCopyOnSelect",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:misc-default-working-dir:
    **/
@@ -865,6 +935,16 @@ terminal_preferences_class_init (TerminalPreferencesClass *klass)
                             FALSE,
                             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * TerminalPreferences:misc-shift-arrows-scroll:
+   **/
+  preferences_props[PROP_MISC_SLIM_TABS] =
+      g_param_spec_boolean ("misc-slim-tabs",
+                            NULL,
+                            "MiscSlimTabs",
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
   /**
    * TerminalPreferences:scrolling-bar:
    **/
