@@ -108,6 +108,7 @@ static gchar    **terminal_screen_get_child_environment         (TerminalScreen
 static void       terminal_screen_update_background             (TerminalScreen        *screen);
 static void       terminal_screen_update_binding_backspace      (TerminalScreen        *screen);
 static void       terminal_screen_update_binding_delete         (TerminalScreen        *screen);
+static void       terminal_screen_update_binding_ambiguous_width(TerminalScreen        *screen);
 static void       terminal_screen_update_encoding               (TerminalScreen        *screen);
 static void       terminal_screen_update_colors                 (TerminalScreen        *screen);
 static void       terminal_screen_update_misc_bell              (TerminalScreen        *screen);
@@ -286,6 +287,7 @@ terminal_screen_init (TerminalScreen *screen)
   /* apply current settings */
   terminal_screen_update_binding_backspace (screen);
   terminal_screen_update_binding_delete (screen);
+  terminal_screen_update_binding_ambiguous_width (screen);
   terminal_screen_update_encoding (screen);
   terminal_screen_update_font (screen);
   terminal_screen_update_misc_bell (screen);
@@ -490,14 +492,18 @@ terminal_screen_draw (GtkWidget *widget,
 
   cairo_save (cr);
 
+  /* draw background image; cairo_set_operator() allows PNG transparency */
   gdk_cairo_set_source_pixbuf (cr, image, 0, 0);
+  cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
   cairo_paint (cr);
   g_object_unref (G_OBJECT (image));
 
+  /* draw vte terminal */
   surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
   ctx = cairo_create (surface);
   gtk_widget_draw (screen->terminal, ctx);
   cairo_set_source_surface (cr, surface, 0, 0);
+  cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
   cairo_paint (cr);
   cairo_destroy (ctx);
   cairo_surface_destroy (surface);
@@ -533,6 +539,8 @@ terminal_screen_preferences_changed (TerminalPreferences *preferences,
     terminal_screen_update_binding_backspace (screen);
   else if (strcmp ("binding-delete", name) == 0)
     terminal_screen_update_binding_delete (screen);
+  else if (strcmp ("binding-ambiguous-width", name) == 0)
+    terminal_screen_update_binding_ambiguous_width (screen);
   else if (strncmp ("color-", name, strlen ("color-")) == 0)
     terminal_screen_update_colors (screen);
   else if (strncmp ("font-", name, strlen ("font-")) == 0)
@@ -574,7 +582,9 @@ terminal_screen_get_child_command (TerminalScreen   *screen,
   struct passwd *pw;
   const gchar   *shell_name;
   const gchar   *shell_fullpath = NULL;
+  gchar         *custom_command = NULL;
   gboolean       command_login_shell;
+  gboolean       run_custom_command;
   guint          i;
   const gchar   *shells[] = { "/bin/sh",
                               "/bin/bash", "/usr/bin/bash",
@@ -591,68 +601,82 @@ terminal_screen_get_child_command (TerminalScreen   *screen,
     }
   else
     {
-      /* use the SHELL environement variable if we're in
-       * non-setuid mode and the path is executable */
-      if (geteuid () == getuid ()
-          && getegid () == getgid ())
+      g_object_get (G_OBJECT (screen->preferences),
+                    "command-login-shell", &command_login_shell,
+                    "run-custom-command", &run_custom_command,
+                    NULL);
+
+      if (run_custom_command)
         {
-          shell_fullpath = g_getenv ("SHELL");
-          if (shell_fullpath != NULL
-              && g_access (shell_fullpath, X_OK) != 0)
-            shell_fullpath = NULL;
+          /* use custom command specified in preferences */
+          g_object_get (G_OBJECT (screen->preferences),
+                        "custom-command", &custom_command,
+                        NULL);
+          shell_fullpath = custom_command;
         }
-
-      if (shell_fullpath == NULL)
+      else
         {
-          pw = getpwuid (getuid ());
-          if (pw != NULL
-              && pw->pw_shell != NULL
-              && g_access (pw->pw_shell, X_OK) == 0)
+          /* use the SHELL environement variable if we're in
+          * non-setuid mode and the path is executable */
+          if (geteuid () == getuid ()
+              && getegid () == getgid ())
             {
-              /* set the shell from the password database */
-              shell_fullpath = pw->pw_shell;
+              shell_fullpath = g_getenv ("SHELL");
+              if (shell_fullpath != NULL
+                  && g_access (shell_fullpath, X_OK) != 0)
+                shell_fullpath = NULL;
             }
-          else
-            {
-              /* lookup a good fallback */
-              for (i = 0; i < G_N_ELEMENTS (shells); i++)
-                {
-                  if (access (shells [i], X_OK) == 0)
-                    {
-                      shell_fullpath = shells [i];
-                      break;
-                    }
-                }
 
-              if (G_UNLIKELY (shell_fullpath == NULL))
+          if (shell_fullpath == NULL)
+            {
+              pw = getpwuid (getuid ());
+              if (pw != NULL
+                  && pw->pw_shell != NULL
+                  && g_access (pw->pw_shell, X_OK) == 0)
                 {
-                  /* the system is truly broken */
-                  g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL,
-                               _("Unable to determine your login shell."));
-                  return FALSE;
+                  /* set the shell from the password database */
+                  shell_fullpath = pw->pw_shell;
                 }
+              else
+                {
+                  /* lookup a good fallback */
+                  for (i = 0; i < G_N_ELEMENTS (shells); i++)
+                    {
+                      if (access (shells [i], X_OK) == 0)
+                        {
+                          shell_fullpath = shells [i];
+                          break;
+                        }
+                    }
+
+                  if (G_UNLIKELY (shell_fullpath == NULL))
+                    {
+                      /* the system is truly broken */
+                      g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL,
+                                   _("Unable to determine your login shell."));
+                      return FALSE;
+                    }
+                 }
             }
         }
 
       terminal_assert (shell_fullpath != NULL);
       shell_name = strrchr (shell_fullpath, '/');
-
       if (shell_name != NULL)
         ++shell_name;
       else
         shell_name = shell_fullpath;
       *command = g_strdup (shell_fullpath);
 
-      g_object_get (G_OBJECT (screen->preferences),
-                    "command-login-shell", &command_login_shell,
-                    NULL);
-
       *argv = g_new (gchar *, 2);
       if (command_login_shell)
         (*argv)[0] = g_strconcat ("-", shell_name, NULL);
       else
         (*argv)[0] = g_strdup (shell_name);
       (*argv)[1] = NULL;
+
+      if (custom_command != NULL)
+        g_free (custom_command);
     }
 
   return TRUE;
@@ -884,6 +908,18 @@ terminal_screen_update_binding_delete (TerminalScreen *screen)
 
 
 
+static void
+terminal_screen_update_binding_ambiguous_width (TerminalScreen *screen)
+{
+  TerminalAmbiguousWidthBinding binding;
+
+  g_object_get (G_OBJECT (screen->preferences), "binding-ambiguous-width", &binding, NULL);
+  vte_terminal_set_cjk_ambiguous_width (VTE_TERMINAL (screen->terminal),
+      binding == TERMINAL_AMBIGUOUS_WIDTH_BINDING_NARROW ? 1 : 2);
+}
+
+
+
 static void
 terminal_screen_update_encoding (TerminalScreen *screen)
 {
@@ -1006,10 +1042,12 @@ terminal_screen_update_colors (TerminalScreen *screen)
 
   /* selection color */
   if (!selection_use_default)
-    selection_use_default = !terminal_preferences_get_color (screen->preferences, "color-selection", &selection);
-  vte_terminal_set_color_highlight (VTE_TERMINAL (screen->terminal), selection_use_default ? NULL : &selection);
-  vte_terminal_set_color_highlight_foreground (VTE_TERMINAL (screen->terminal),
-                                               selection_use_default && has_bg ? NULL : &bg);
+    {
+      selection_use_default = !terminal_preferences_get_color (screen->preferences, "color-selection", &selection);
+      vte_terminal_set_color_highlight_foreground (VTE_TERMINAL (screen->terminal), selection_use_default ? NULL : &selection);
+      selection_use_default = !terminal_preferences_get_color (screen->preferences, "color-selection-bg", &selection);
+      vte_terminal_set_color_highlight (VTE_TERMINAL (screen->terminal), selection_use_default ? NULL : &selection);
+    }
 
   /* bold color */
   if (!bold_use_default)
@@ -1195,9 +1233,17 @@ static void
 terminal_screen_vte_selection_changed (VteTerminal    *terminal,
                                        TerminalScreen *screen)
 {
+  gboolean copy_on_select;
+
   terminal_return_if_fail (VTE_IS_TERMINAL (terminal));
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
+  /* copy vte selection to GDK_SELECTION_CLIPBOARD if option is set */
+  g_object_get (G_OBJECT (screen->preferences),
+                "misc-copy-on-select", &copy_on_select, NULL);
+  if (copy_on_select)
+    vte_terminal_copy_clipboard (terminal);
+
   g_signal_emit (G_OBJECT (screen), screen_signals[SELECTION_CHANGED], 0);
 }
 
@@ -1674,46 +1720,63 @@ terminal_screen_get_geometry (TerminalScreen *screen,
  * I don't like this way, but its required to work-around a Gtk+
  * bug (maybe also caused by a Vte bug, not sure).
  *
- * Code borrowed from gnome-terminal (terminal_window_update_geometry).
+ * Code for GTK > 3.19.5 borrowed from gnome-terminal
+ * (terminal_window_update_geometry).
  **/
 void
 terminal_screen_set_window_geometry_hints (TerminalScreen *screen,
                                            GtkWindow      *window)
 {
   GdkGeometry    hints;
+  glong          char_width, char_height;
+#if GTK_CHECK_VERSION (3, 19, 5)
   GtkRequisition vbox_request;
   GtkAllocation  toplevel_allocation, vbox_allocation;
-  glong          char_width, char_height;
   glong          grid_width, grid_height;
   glong          chrome_width, chrome_height;
   gint           csd_width, csd_height;
+#else
+  gint           xpad, ypad;
+#endif
 
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   terminal_return_if_fail (VTE_IS_TERMINAL (screen->terminal));
   terminal_return_if_fail (gtk_widget_get_realized (screen));
   terminal_return_if_fail (gtk_widget_get_realized (window));
 
+#if GTK_CHECK_VERSION (3, 19, 5)
   terminal_screen_get_geometry (screen, &char_width, &char_height, NULL, NULL);
   terminal_screen_get_size (screen, &grid_width, &grid_height);
 
-  gtk_widget_get_preferred_size (TERMINAL_WINDOW (window)->vbox, NULL, &vbox_request);
+  gtk_widget_get_preferred_size (terminal_window_get_vbox (TERMINAL_WINDOW (window)), NULL, &vbox_request);
   chrome_width = vbox_request.width - (char_width * grid_width);
   chrome_height = vbox_request.height - (char_height * grid_height);
 
-  gtk_widget_get_allocation (TERMINAL_WINDOW (window)->vbox, &vbox_allocation);
+  gtk_widget_get_allocation (terminal_window_get_vbox (TERMINAL_WINDOW (window)), &vbox_allocation);
   gtk_widget_get_allocation (GTK_WIDGET (window), &toplevel_allocation);
   csd_width = toplevel_allocation.width - vbox_allocation.width;
   csd_height = toplevel_allocation.height - vbox_allocation.height;
 
   hints.base_width = chrome_width + csd_width;
   hints.base_height = chrome_height + csd_height;
+#else
+  terminal_screen_get_geometry (screen, &char_width, &char_height, &xpad, &ypad);
+
+  hints.base_width = xpad;
+  hints.base_height = ypad;
+#endif
+
   hints.width_inc = char_width;
   hints.height_inc = char_height;
   hints.min_width = hints.base_width + hints.width_inc * 4;
   hints.min_height = hints.base_height + hints.height_inc * 2;
 
   gtk_window_set_geometry_hints (window,
+#if GTK_CHECK_VERSION (3, 19, 5)
                                  NULL,
+#else
+                                 screen->terminal,
+#endif
                                  &hints,
                                  GDK_HINT_RESIZE_INC
                                  | GDK_HINT_MIN_SIZE
@@ -2186,6 +2249,8 @@ terminal_screen_set_encoding (TerminalScreen *screen,
                               const gchar    *charset)
 {
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  if (charset == NULL)
+    g_get_charset (&charset);
   if (!vte_terminal_set_encoding (VTE_TERMINAL (screen->terminal), charset, NULL)) {
     g_printerr("Failed to set encoding %s\n", charset);
   }
@@ -2247,7 +2312,7 @@ terminal_screen_update_scrolling_bar (TerminalScreen *screen)
 
   toplevel = gtk_widget_get_toplevel (GTK_WIDGET (screen));
   if (TERMINAL_IS_WINDOW (toplevel))
-    visibility = TERMINAL_WINDOW (toplevel)->scrollbar_visibility;
+    visibility = terminal_window_get_scrollbar_visibility (TERMINAL_WINDOW (toplevel));
 
   if (G_LIKELY (visibility == TERMINAL_VISIBILITY_DEFAULT))
     {
@@ -2289,21 +2354,31 @@ terminal_screen_update_scrolling_bar (TerminalScreen *screen)
 void
 terminal_screen_update_font (TerminalScreen *screen)
 {
-  gboolean               font_allow_bold;
-  gchar                 *font_name;
-  PangoFontDescription  *font_desc;
-  glong                  grid_w = 0, grid_h = 0;
-  GtkWidget             *toplevel;
+  gboolean              font_use_system, font_allow_bold;
+  gchar                *font_name;
+  PangoFontDescription *font_desc;
+  glong                 grid_w = 0, grid_h = 0;
+  GtkWidget            *toplevel;
+  GSettings            *settings;
 
   terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   terminal_return_if_fail (TERMINAL_IS_PREFERENCES (screen->preferences));
   terminal_return_if_fail (VTE_IS_TERMINAL (screen->terminal));
 
   g_object_get (G_OBJECT (screen->preferences),
+                "font-use-system", &font_use_system,
                 "font-allow-bold", &font_allow_bold,
-                "font-name", &font_name,
                 NULL);
 
+  if (font_use_system)
+    {
+      settings = g_settings_new ("org.gnome.desktop.interface");
+      font_name = g_settings_get_string (settings, "monospace-font-name");
+      g_object_unref (settings);
+    }
+  else
+    g_object_get (G_OBJECT (screen->preferences), "font-name", &font_name, NULL);
+
   toplevel = gtk_widget_get_toplevel (GTK_WIDGET (screen));
   if (TERMINAL_IS_WINDOW (toplevel))
     {
