@@ -84,6 +84,8 @@ static void     terminal_window_dropdown_toggle_real             (TerminalWindow
                                                                   guint32                 timestamp,
                                                                   gboolean                force_show);
 static void     terminal_window_dropdown_update_geometry         (TerminalWindowDropdown *dropdown);
+static void     terminal_dropdown_window_screen_size_changed     (GdkScreen              *screen,
+                                                                  TerminalWindowDropdown *dropdown);
 
 
 struct _TerminalWindowDropdownClass
@@ -399,6 +401,11 @@ terminal_window_dropdown_finalize (GObject *object)
   if (dropdown->status_icon != NULL)
     g_object_unref (G_OBJECT (dropdown->status_icon));
 
+  if (dropdown->screen != NULL)
+    g_signal_handlers_disconnect_by_func (G_OBJECT (dropdown->screen),
+                                          G_CALLBACK (terminal_dropdown_window_screen_size_changed),
+                                          dropdown);
+
   (*G_OBJECT_CLASS (terminal_window_dropdown_parent_class)->finalize) (object);
 }
 
@@ -700,7 +707,7 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
   TerminalWindow    *window = TERMINAL_WINDOW (dropdown);
   gint               w, h;
   GdkRectangle       monitor_geo;
-  gint               x_dest, y_dest;
+  gint               x_dest, y_dest, x_pos, y_pos;
   GtkRequisition     req1;
   gboolean           move_to_active;
   gboolean           keep_above;
@@ -728,7 +735,18 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
   if (move_to_active
       || dropdown->screen == NULL
       || dropdown->monitor_num == -1)
-    dropdown->screen = xfce_gdk_screen_get_active (&dropdown->monitor_num);
+    {
+      if (dropdown->screen != NULL)
+        g_signal_handlers_disconnect_by_func (G_OBJECT (dropdown->screen),
+                                              G_CALLBACK (terminal_dropdown_window_screen_size_changed),
+                                              dropdown);
+
+      dropdown->screen = xfce_gdk_screen_get_active (&dropdown->monitor_num);
+
+      /* watch for screen size changes to update terminal geometry accordingly*/
+      g_signal_connect (G_OBJECT (dropdown->screen), "size-changed",
+                        G_CALLBACK (terminal_dropdown_window_screen_size_changed), dropdown);
+    }
 
   /* get the active monitor size */
 #if GTK_CHECK_VERSION (3, 22, 0)
@@ -786,12 +804,18 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
   x_dest = monitor_geo.x + (monitor_geo.width - w) * dropdown->rel_position;
   y_dest = monitor_geo.y;
 
+  /* move */
+  gtk_window_move (GTK_WINDOW (dropdown), x_dest, y_dest);
+
   /* show window */
   if (!visible)
     gtk_window_present_with_time (GTK_WINDOW (dropdown), timestamp);
 
-  /* move */
-  gtk_window_move (GTK_WINDOW (dropdown), x_dest, y_dest);
+  /* check window position after showing it
+   * https://bugzilla.xfce.org/show_bug.cgi?id=10713 */
+  gtk_window_get_position (GTK_WINDOW (dropdown), &x_pos, &y_pos);
+  if (x_pos != x_dest || y_pos != y_dest)
+    gtk_window_move (GTK_WINDOW (dropdown), x_dest, y_dest);
 
   /* force focus to the window */
   terminal_util_activate_window (GTK_WINDOW (dropdown));
@@ -900,6 +924,16 @@ terminal_window_dropdown_get_timestamp (GtkWidget   *widget,
 
 
 
+static void
+terminal_dropdown_window_screen_size_changed (GdkScreen              *screen,
+                                              TerminalWindowDropdown *dropdown)
+{
+  /* resize/move terminal window due to a screen size change */
+  terminal_window_dropdown_show (dropdown, 0);
+}
+
+
+
 GtkWidget *
 terminal_window_dropdown_new (const gchar        *role,
                               const gchar        *icon,
